#![feature(round_char_boundary)]
use crate::parser::document;
use tracing_subscriber::fmt::format::FmtSpan;
use tracing_subscriber::EnvFilter;
mod parser;

const TEST_DOC: &'static str = include_str!("../toy_language.txt");

fn main() -> Result<(), Box<dyn std::error::Error>> {
    init_telemetry()?;
    let parsed = document(TEST_DOC);
    println!("{}\n\n\n", TEST_DOC);
    println!("{:#?}", parsed);
    Ok(())
}

fn init_telemetry() -> Result<(), Box<dyn std::error::Error>> {
    let env_filter = EnvFilter::try_from_default_env().unwrap_or(EnvFilter::new("WARN"));
    let format = tracing_subscriber::fmt::format()
        .pretty()
        .with_file(true)
        .with_line_number(true)
        .with_thread_ids(false)
        .with_target(false);
    let subscriber = tracing_subscriber::fmt()
        .event_format(format)
        .with_span_events(FmtSpan::ENTER | FmtSpan::EXIT)
        .with_env_filter(env_filter)
        .finish();
    tracing::subscriber::set_global_default(subscriber)?;
    Ok(())
}
